"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiteSessionAuth = SiteSessionAuth;
const biHeaderGenerator_js_1 = require("../bi/biHeaderGenerator.js");
const common_js_1 = require("../common.js");
const tokenHelpers_js_1 = require("../tokenHelpers.js");
const types_js_1 = require("./oauth2/types.js");
const pre_warm_cookie_js_1 = require("./pre-warm-cookie.js");
function SiteSessionAuth(config) {
    const _tokens = config.tokens || {
        accessToken: { value: '', expiresAt: 0 },
        refreshToken: { value: '', role: types_js_1.TokenRole.NONE },
    };
    const setTokens = (tokens) => {
        _tokens.accessToken = tokens.accessToken;
        _tokens.refreshToken = tokens.refreshToken;
    };
    const getAuthHeaders = async () => {
        if (!_tokens.accessToken?.value || (0, tokenHelpers_js_1.isTokenExpired)(_tokens.accessToken)) {
            const tokens = await generateVisitorTokens({
                refreshToken: _tokens.refreshToken,
            });
            setTokens(tokens);
        }
        return Promise.resolve({
            headers: { Authorization: _tokens.accessToken.value },
        });
    };
    const generateVisitorTokens = async (tokens) => {
        if (tokens?.accessToken?.value &&
            tokens?.refreshToken?.value &&
            !(0, tokenHelpers_js_1.isTokenExpired)(tokens.accessToken)) {
            return tokens;
        }
        if (tokens?.refreshToken?.value) {
            try {
                const newTokens = await renewToken(tokens.refreshToken);
                return newTokens;
            }
            catch (e) {
                // just continue and create a visitor one
            }
        }
        const tokensResponse = await fetchTokens({
            clientId: config.clientId,
            grantType: 'anonymous',
        });
        return {
            accessToken: (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in),
            refreshToken: {
                value: tokensResponse.refresh_token,
                role: types_js_1.TokenRole.VISITOR,
            },
        };
    };
    const renewToken = async (refreshToken) => {
        const tokensResponse = await fetchTokens({
            refreshToken: refreshToken.value,
            grantType: 'refresh_token',
        });
        const accessToken = (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in);
        return {
            accessToken,
            refreshToken,
        };
    };
    const loggedIn = () => {
        return _tokens.refreshToken.role === types_js_1.TokenRole.MEMBER;
    };
    return {
        generateVisitorTokens,
        renewToken,
        getAuthHeaders,
        setTokens,
        loggedIn,
        getTokens: () => _tokens,
        isSessionSynced: pre_warm_cookie_js_1.isVisitorCookieWarmedUp,
        syncToWixPages: pre_warm_cookie_js_1.preWarmVisitorCookie,
        shouldUseCDN: true,
    };
}
const fetchTokens = async (payload, headers = {}) => {
    const res = await fetch(`https://${common_js_1.DEFAULT_API_URL}/oauth2/token`, {
        method: 'POST',
        body: JSON.stringify(payload),
        headers: {
            ...(0, biHeaderGenerator_js_1.biHeaderGenerator)({
                entityFqdn: 'wix.identity.oauth.v1.refresh_token',
                methodFqn: 'wix.identity.oauth2.v1.Oauth2Ng.Token',
                packageName: '@wix/sdk',
            }),
            'Content-Type': 'application/json',
            ...headers,
        },
    });
    if (res.status !== 200) {
        let responseJson;
        try {
            responseJson = await res.json();
        }
        catch { }
        throw new Error(`Failed to fetch tokens from OAuth API: ${res.statusText}. request id: ${res.headers.get('x-request-id')}. ${responseJson ? `Response: ${JSON.stringify(responseJson)}` : ''}`);
    }
    const json = await res.json();
    return json;
};
