"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OAuthStrategy = OAuthStrategy;
const wixClient_js_1 = require("../../wixClient.js");
const redirects_1 = require("@wix/redirects");
const tokenHelpers_js_1 = require("../../tokenHelpers.js");
const identity_1 = require("@wix/identity");
const common_js_1 = require("../../common.js");
const types_js_1 = require("./types.js");
const iframeUtils_js_1 = require("../../iframeUtils.js");
const constants_js_1 = require("./constants.js");
const biHeaderGenerator_js_1 = require("../../bi/biHeaderGenerator.js");
const pkce_challenge_js_1 = require("./pkce-challenge.js");
const pre_warm_cookie_js_1 = require("../pre-warm-cookie.js");
const moduleWithTokens = { redirects: redirects_1.redirects, authentication: identity_1.authentication, recovery: identity_1.recovery, verification: identity_1.verification };
function OAuthStrategy(config) {
    const _tokens = config.tokens || {
        accessToken: { value: '', expiresAt: 0 },
        refreshToken: { value: '', role: types_js_1.TokenRole.NONE },
    };
    const setTokens = (tokens) => {
        _tokens.accessToken = tokens.accessToken;
        _tokens.refreshToken = tokens.refreshToken;
    };
    let _state = {
        loginState: types_js_1.LoginState.INITIAL,
    };
    const getAuthHeaders = async () => {
        if (!_tokens.accessToken?.value || (0, tokenHelpers_js_1.isTokenExpired)(_tokens.accessToken)) {
            const tokens = await generateVisitorTokens({
                refreshToken: _tokens.refreshToken,
            });
            setTokens(tokens);
        }
        return Promise.resolve({
            headers: { Authorization: _tokens.accessToken.value },
        });
    };
    const wixClientWithTokens = (0, wixClient_js_1.createClient)({
        modules: moduleWithTokens,
        auth: { getAuthHeaders },
    });
    const generateVisitorTokens = async (tokens) => {
        if (tokens?.accessToken?.value &&
            tokens?.refreshToken?.value &&
            !(0, tokenHelpers_js_1.isTokenExpired)(tokens.accessToken)) {
            return tokens;
        }
        if (tokens?.refreshToken?.value) {
            try {
                const newTokens = await renewToken(tokens.refreshToken);
                return newTokens;
            }
            catch (e) {
                // just continue and create a visitor one
            }
        }
        const tokensResponse = await fetchTokens({
            clientId: config.clientId,
            grantType: 'anonymous',
        });
        return {
            accessToken: (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in),
            refreshToken: {
                value: tokensResponse.refresh_token,
                role: types_js_1.TokenRole.VISITOR,
            },
        };
    };
    const renewToken = async (refreshToken) => {
        const tokensResponse = await fetchTokens({
            refreshToken: refreshToken.value,
            grantType: 'refresh_token',
        });
        const accessToken = (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in);
        return {
            accessToken,
            refreshToken,
        };
    };
    const generatePKCE = () => {
        const pkceState = (0, pkce_challenge_js_1.pkceChallenge)();
        return {
            codeChallenge: pkceState.code_challenge,
            codeVerifier: pkceState.code_verifier,
            state: (0, pkce_challenge_js_1.pkceChallenge)().code_challenge,
        };
    };
    const generateOAuthData = (redirectUri, originalUri) => {
        const state = { redirectUri };
        const pkceState = generatePKCE();
        return {
            ...state,
            originalUri: originalUri ?? '',
            codeChallenge: pkceState.codeChallenge,
            codeVerifier: pkceState.codeVerifier,
            state: (0, pkce_challenge_js_1.pkceChallenge)().code_challenge,
        };
    };
    const getAuthorizationUrlWithOptions = async (oauthData, responseMode, prompt, sessionToken) => {
        const { redirectSession } = await wixClientWithTokens.redirects.createRedirectSession({
            auth: {
                authRequest: {
                    redirectUri: oauthData.redirectUri,
                    ...(oauthData.redirectUri && {
                        redirectUri: oauthData.redirectUri,
                    }),
                    clientId: config.clientId,
                    codeChallenge: oauthData.codeChallenge,
                    codeChallengeMethod: 'S256',
                    responseMode,
                    responseType: 'code',
                    scope: 'offline_access',
                    state: oauthData.state,
                    ...(sessionToken && { sessionToken }),
                },
                prompt: redirects_1.redirects.Prompt[prompt],
            },
        });
        return {
            authUrl: redirectSession.fullUrl,
            authorizationEndpoint: redirectSession.urlDetails.endpoint,
            sessionToken: redirectSession.sessionToken,
        };
    };
    const getAuthUrl = async (oauthData = generateOAuthData('unused://'), opts = {
        prompt: 'login',
    }) => {
        return getAuthorizationUrlWithOptions(oauthData, opts.responseMode ?? 'fragment', opts.prompt ?? 'login', opts.sessionToken);
    };
    const parseFromUrl = (url, responseMode = 'fragment') => {
        const parsedUrl = new URL(url ?? window.location.href);
        const params = responseMode === 'query'
            ? parsedUrl.searchParams
            : new URLSearchParams(parsedUrl.hash.substring(1));
        const code = params.get('code');
        const state = params.get('state');
        const error = params.get('error');
        const errorDescription = params.get('error_description');
        return { code, state, ...(error && { error, errorDescription }) };
    };
    const getMemberTokens = async (code, state, oauthData) => {
        if (!code || !state) {
            throw new Error('Missing code or _state');
        }
        else if (state !== oauthData.state) {
            throw new Error('Invalid _state');
        }
        const tokensResponse = await fetchTokens({
            clientId: config.clientId,
            grantType: 'authorization_code',
            ...(oauthData.redirectUri && { redirectUri: oauthData.redirectUri }),
            code,
            codeVerifier: oauthData.codeVerifier,
        });
        return {
            accessToken: (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in),
            refreshToken: {
                value: tokensResponse.refresh_token,
                role: types_js_1.TokenRole.MEMBER,
            },
        };
    };
    const logout = async (originalUrl) => {
        const { redirectSession } = await wixClientWithTokens.redirects.createRedirectSession({
            logout: { clientId: config.clientId },
            callbacks: {
                postFlowUrl: originalUrl,
            },
        });
        _tokens.accessToken = { value: '', expiresAt: 0 };
        _tokens.refreshToken = { value: '', role: types_js_1.TokenRole.NONE };
        return { logoutUrl: redirectSession.fullUrl };
    };
    const handleState = (response) => {
        if (response.state === identity_1.authentication.StateType.SUCCESS) {
            return {
                loginState: types_js_1.LoginState.SUCCESS,
                data: { sessionToken: response.sessionToken },
            };
        }
        else if (response.state === identity_1.authentication.StateType.REQUIRE_OWNER_APPROVAL) {
            return {
                loginState: types_js_1.LoginState.OWNER_APPROVAL_REQUIRED,
            };
        }
        else if (response.state === identity_1.authentication.StateType.REQUIRE_EMAIL_VERIFICATION) {
            _state = {
                loginState: types_js_1.LoginState.EMAIL_VERIFICATION_REQUIRED,
                data: { stateToken: response.stateToken },
            };
            return _state;
        }
        return {
            loginState: types_js_1.LoginState.FAILURE,
            error: 'Unknown _state',
        };
    };
    const register = async (params) => {
        try {
            const res = await wixClientWithTokens.authentication.registerV2({
                email: params.email,
            }, {
                password: params.password,
                profile: params.profile,
                ...(params.captchaTokens && {
                    captchaTokens: [
                        {
                            Recaptcha: params.captchaTokens?.recaptchaToken,
                            InvisibleRecaptcha: params.captchaTokens?.invisibleRecaptchaToken,
                        },
                    ],
                }),
            });
            return handleState(res);
        }
        catch (e) {
            const emailValidation = e.details.validationError?.fieldViolations?.find((v) => v.data.type === 'EMAIL');
            if (emailValidation) {
                return {
                    loginState: types_js_1.LoginState.FAILURE,
                    error: emailValidation.description,
                    errorCode: 'invalidEmail',
                };
            }
            if (e.details.applicationError?.code === constants_js_1.MISSING_CAPTCHA) {
                return {
                    loginState: types_js_1.LoginState.FAILURE,
                    error: e.message,
                    errorCode: 'missingCaptchaToken',
                };
            }
            if (e.details.applicationError?.code === constants_js_1.EMAIL_EXISTS) {
                return {
                    loginState: types_js_1.LoginState.FAILURE,
                    error: e.message,
                    errorCode: 'emailAlreadyExists',
                };
            }
            if (e.details.applicationError?.code === constants_js_1.INVALID_CAPTCHA) {
                return {
                    loginState: types_js_1.LoginState.FAILURE,
                    error: e.message,
                    errorCode: 'invalidCaptchaToken',
                };
            }
            return {
                loginState: types_js_1.LoginState.FAILURE,
                error: e.message,
            };
        }
    };
    const login = async (params) => {
        try {
            const res = await wixClientWithTokens.authentication.loginV2({
                email: params.email,
            }, {
                password: params.password,
                ...(params.captchaTokens && {
                    captchaTokens: [
                        {
                            Recaptcha: params.captchaTokens?.recaptchaToken,
                            InvisibleRecaptcha: params.captchaTokens?.invisibleRecaptchaToken,
                        },
                    ],
                }),
            });
            return handleState(res);
        }
        catch (e) {
            return {
                loginState: types_js_1.LoginState.FAILURE,
                error: e.message,
                errorCode: e.details.applicationError?.code === constants_js_1.MISSING_CAPTCHA
                    ? 'missingCaptchaToken'
                    : e.details.applicationError?.code === constants_js_1.INVALID_CAPTCHA
                        ? 'invalidCaptchaToken'
                        : e.details.applicationError.code === constants_js_1.INVALID_PASSWORD
                            ? 'invalidPassword'
                            : e.details.applicationError.code === constants_js_1.RESET_PASSWORD
                                ? 'resetPassword'
                                : 'invalidEmail',
            };
        }
    };
    const processVerification = async (nextInputs, state) => {
        const stateToUse = state ?? _state;
        if (stateToUse.loginState === types_js_1.LoginState.EMAIL_VERIFICATION_REQUIRED) {
            const code = nextInputs.verificationCode ?? nextInputs.code;
            const res = await wixClientWithTokens.verification.verifyDuringAuthentication(code, { stateToken: stateToUse.data.stateToken });
            return handleState(res);
        }
        return {
            loginState: types_js_1.LoginState.FAILURE,
            error: 'Unknown _state',
        };
    };
    const getMemberTokensForDirectLogin = async (sessionToken) => {
        const oauthPKCE = generatePKCE();
        const { authUrl } = await getAuthorizationUrlWithOptions(oauthPKCE, 'web_message', 'none', sessionToken);
        const iframePromise = (0, iframeUtils_js_1.addPostMessageListener)(oauthPKCE.state);
        const iframeEl = (0, iframeUtils_js_1.loadFrame)(authUrl);
        return iframePromise
            .then((res) => {
            return getMemberTokens(res.code, res.state, oauthPKCE);
        })
            .finally(() => {
            if (document.body.contains(iframeEl)) {
                iframeEl.parentElement?.removeChild(iframeEl);
            }
        });
    };
    const sendPasswordResetEmail = async (email, redirectUri) => {
        await wixClientWithTokens.recovery.sendRecoveryEmail(email, {
            redirect: { url: redirectUri, clientId: config.clientId },
        });
    };
    const loggedIn = () => {
        return _tokens.refreshToken.role === types_js_1.TokenRole.MEMBER;
    };
    const getMemberTokensForExternalLogin = async (memberId, apiKey) => {
        const tokensResponse = await fetchTokens({
            grant_type: 'authorized_client',
            scope: 'offline_access',
            member_id: memberId,
        }, {
            Authorization: _tokens.accessToken.value + ',' + apiKey,
        });
        return {
            accessToken: (0, tokenHelpers_js_1.createAccessToken)(tokensResponse.access_token, tokensResponse.expires_in),
            refreshToken: {
                value: tokensResponse.refresh_token,
                role: types_js_1.TokenRole.MEMBER,
            },
        };
    };
    return {
        generateVisitorTokens,
        renewToken,
        parseFromUrl,
        getAuthUrl,
        getMemberTokens,
        generateOAuthData,
        getAuthHeaders,
        setTokens,
        getTokens: () => _tokens,
        loggedIn,
        logout,
        register,
        processVerification,
        login,
        getMemberTokensForDirectLogin,
        getMemberTokensForExternalLogin,
        sendPasswordResetEmail,
        captchaInvisibleSiteKey: '6LdoPaUfAAAAAJphvHoUoOob7mx0KDlXyXlgrx5v',
        captchaVisibleSiteKey: '6Ld0J8IcAAAAANyrnxzrRlX1xrrdXsOmsepUYosy',
        sessions: {
            isSessionSynced: pre_warm_cookie_js_1.isVisitorCookieWarmedUp,
            syncToWixPages: pre_warm_cookie_js_1.preWarmVisitorCookie,
        },
        shouldUseCDN: true,
    };
}
const fetchTokens = async (payload, headers = {}) => {
    const res = await fetch(`https://${common_js_1.DEFAULT_API_URL}/oauth2/token`, {
        method: 'POST',
        body: JSON.stringify(payload),
        headers: {
            ...(0, biHeaderGenerator_js_1.biHeaderGenerator)({
                entityFqdn: 'wix.identity.oauth.v1.refresh_token',
                methodFqn: 'wix.identity.oauth2.v1.Oauth2Ng.Token',
                packageName: '@wix/sdk',
            }),
            'Content-Type': 'application/json',
            ...headers,
        },
    });
    if (res.status !== 200) {
        let responseJson;
        try {
            responseJson = await res.json();
        }
        catch { }
        throw new Error(`Failed to fetch tokens from OAuth API: ${res.statusText}. request id: ${res.headers.get('x-request-id')}. ${responseJson ? `Response: ${JSON.stringify(responseJson)}` : ''}`);
    }
    const json = await res.json();
    return json;
};
